<?php
/*
 * PHPmyLMB-XT, a Lightweight media browser.
 * Generate a web page with media contents such as videos and audios.
 * Very useful to share with people :)
 * Cross-forked with PHPmyLMB (https://github.com/benpro/PHPmyLMB)
 * LICENCE is WTFPL.
 */

if (strpos($_SERVER['REMOTE_ADDR'], '.') !== FALSE) {
   header('X-IPv6: FALSE');
   header('X-IP_I_see: '.$_SERVER['REMOTE_ADDR']);
   }
else {
   header('X-IPv6: TRUE');
   header('X-IP_I_see: '.$_SERVER['REMOTE_ADDR']);
   }

function serverUrl($return_subfolder = false) // return full URL for feed
{
    $https = (!empty($_SERVER['HTTPS']) && (strtolower($_SERVER['HTTPS'])=='on')) || $_SERVER["SERVER_PORT"]=='443'; // HTTPS detection.
    $serverport = ($_SERVER["SERVER_PORT"]=='80' || ($https && $_SERVER["SERVER_PORT"]=='443') ? '' : ':'.$_SERVER["SERVER_PORT"]);
    if ($return_subfolder === true) {
        $path = pathinfo( $_SERVER['PHP_SELF'] );
        $finalslash = ( $path['dirname'] != '/' ) ? '/' : '';
        $subfolder = $path['dirname'] . $finalslash;
    } else $subfolder = '';
    while (substr($subfolder, -1) === '/')
     $subfolder = substr($subfolder, 0, -1);

    return 'http'.($https?'s':'').'://'.$_SERVER["SERVER_NAME"].$serverport.$subfolder;
}

require 'config.default.php';
if (file_exists('config.php'))
 require 'config.php';

/**
 * @desc Construct an array with all files found.
 *
 * @param string $sort Mode used for sorting files.
 *  Default: asc
 *  Can be: asc or mtime
 *
 * @return array $filesArray
 */
function getFiles($sort = 'asc')
{
    global $conf;
    /* If results are cached and cache not expired, use it. */

    if ($conf['cache_enabled'] && is_readable($conf['cache_path'].$sort)) {
        if ((time() - filemtime($conf['cache_path'].$sort)) <= $conf['cache_expire']) {
            $fileCache = fopen($conf['cache_path'].$sort, 'r');
            $contents = fread($fileCache, filesize($conf['cache_path'].$sort));
            fclose($fileCache);

            return unserialize($contents);
        }
    }
    /* Search for files (which are allowed_extensions) in all directories. */
    $directories = glob('*', GLOB_ONLYDIR);
    foreach ($directories as $dir) {
        $files = glob('./' . $dir . '/' . $conf['allowed_extensions'], GLOB_BRACE);
        foreach ($files as $file) {
            $name = explode("/", $file);
            $mtime = filemtime($file);
            $filesValues[$mtime] = array(
                'name' => $name[2],
                'mtime' => $mtime,
            );
        }
        if ($sort == 'mtime') {
            krsort($filesValues);
            $filesArray[$dir] = $filesValues;
        } else { // Default to ascending.
            $filesArray[$dir] = $filesValues;
        }
        unset($filesValues);
    }
    /* Store results in the cache & return it. */
    if ($conf['cache_enabled']) {
        $fileCache = fopen($conf['cache_path'].$sort, 'w');
        if ($fileCache !== false) {
            fwrite($fileCache, serialize($filesArray));
            fclose($fileCache);
        } else {
            trigger_error(
                'Cache is enabled but the file used for cache cannot be written!',
                E_USER_WARNING
            );
        }
    }

    return $filesArray;
}

/**
 * @desc Contruct the explorer HTML part.
 * @return string $explorer HTML code to generate.
 */
function explorerHTML()
{
    /* Sort can be only mtime or asc. */
    if (isset($_GET['sort']) && ($_GET['sort'] == 'mtime')) {
        $sort = 'mtime';
    } else {
       $sort = 'asc';
    }
    $filesArray = getFiles($sort);
    $explorer = '';
    foreach ($filesArray as $dirname => $files) {
        $dirnameurlencoded = rawurlencode($dirname);
        $explorer .= <<<EOT

    <div class="vignette">
        <div class="title">$dirname/</div>
            <ul>

EOT;
        foreach ($files as $file) {
            //$filenameurlencoded = rawurlencode($file['name']);    /* This is W3C valid but shits sometimes */
	    $filenameurlencoded = $file['name'];  /* This is not W3C valid but works so fuck it  */
            $explorer .= <<<EOT

                <li>
                    <a href="$dirnameurlencoded/$filenameurlencoded"><img title="Right click → Save as" alt="" src="save.png"></a>
                    <a href="?file=$dirnameurlencoded/$filenameurlencoded&amp;sort=$sort">{$file['name']}</a>
                </li>

EOT;
        }
        $explorer .= <<<EOT

        </ul>
    </div>

EOT;
    }

    return $explorer;
}

/* Feed part. */
if (isset($_GET['feed'])) {
    header('Content-Type: application/atom+xml; charset=UTF-8');
    $date = date('c');
    print <<<EOT
<?xml version="1.0" encoding="utf-8"?>
<feed xmlns="http://www.w3.org/2005/Atom">
    <title type="text">{$conf['title']}</title>
    <link rel="self" type="application/atom+xml" href="{$conf['uri']}/?feed" />
    <id>tag:phpmylmb,2000:1</id>
    <updated>$date</updated>

EOT;
    $filesArray = getFiles();
    foreach ($filesArray as $dirname => $files) {
        foreach ($files as $file) {
            $content='';
            $nameurlencoded = rawurlencode($file['name']);
            $dirnameurlencoded = rawurlencode($dirname);
            $updated = date(DATE_ATOM, $file['mtime']);
            $name = htmlentities($file['name'], ENT_COMPAT | ENT_XML1, 'UTF-8');
            if (preg_match('/(.png|.gif|.jpg|.jpeg|.svg)/i', $file['name']))
              $content = "<content type=\"html\"><![CDATA[<img alt=\"$name\" src=\"{$conf['uri']}/$dirnameurlencoded/$nameurlencoded\"><br>$name]]></content>";

            $entries[$file['mtime']] = <<<EOT

    <entry>
        <title type="html">$dirname/$name</title>
        <link href="{$conf['uri']}/?file=$dirnameurlencoded/$nameurlencoded"/>
        <id>{$conf['uri']}/?file=$dirnameurlencoded/$nameurlencoded</id>
        <updated>$updated</updated>
        <author><name>{$conf['author']}</name></author>
        <summary type="html">$name</summary>
        $content
    </entry>

EOT;
        }
    }
    /* Finally return the last entries ($conf['feed_items']) for the feed.*/
    krsort($entries);
    $entries = array_slice($entries, 0, $conf['feed_items']);
    foreach ($entries as $entry) {
        print $entry;
    }
    print '</feed>';
    exit(0);
}

/* XSPF playlist part. */
if (isset($_GET['playlist'])) {
    (isset($_GET['sort'])) ? $sort = $_GET['sort'] : $sort = 'asc';
    header('Content-Type: application/octet-stream');
    header('Content-Disposition: attachment; filename="playlist.xspf"');
    print <<<EOT
<?xml version="1.0" encoding="UTF-8"?>
<playlist xmlns="http://xspf.org/ns/0/" xmlns:vlc="http://www.videolan.org/vlc/playlist/ns/0/" version="1">
<title>Playlist of {$conf['title']}</title>
    <trackList>
EOT;

    $filesArray = getFiles();
    foreach ($filesArray as $dirname => $files) {
        foreach ($files as $file) {
            if (preg_match('/(.webm|.opus|.ogg)/i', $file['name'])) {
                $nameurlencoded = rawurlencode($file['name']);
                $dirnameurlencoded = rawurlencode($dirname);
                $entries[$file['mtime']] = <<<EOT

            <track>
                <title>$dirname/{$file['name']}</title>
                <location>{$conf['uri']}/$dirnameurlencoded/$nameurlencoded</location>
            </track>

EOT;
            }
        }
    }
    if ($sort == 'mtime') {
        krsort($entries);
    } // Else, default is asc mode.
    foreach ($entries as $entry) {
        print $entry;
    }
    print <<<EOT

    </trackList>
</playlist>
EOT;
    exit(0);
}

/*
 * File viewing part. When user has clicked on a file.
 * Generates the embedded media.
 */
if (isset($_GET['file'])) {
    $path = urldecode($_GET['file']);
    /* Verify if the file exists and construct the embedded media. */
    if (file_exists('./' . $path)) {
        $mtime = filemtime($path);
        $mtimeATOM = date(DATE_ATOM, $mtime);
        $mtimeHuman = date(DATE_RFC822, $mtime);
        $mediatitle = $path;
        $pathinfo = pathinfo($path);
        $pathurlencoded = rawurlencode($pathinfo['dirname']) . '/' . rawurlencode($pathinfo['basename']);
        $mediacode = <<<EOT

            <div class="fileinfo">
                File: <time datetime="$mtimeATOM">$path</time><br />
                Added: $mtimeHuman
            </div>

EOT;
        /* Handle file types. */
	switch($pathinfo['extension']) {
		case 'webm':
			if (file_exists('./'.$path.'.vtt')) // does the video has associated VTT subtitles ?
				$mediacode .= "\t\t\t" . '<video id="media" src="' . $pathurlencoded  .'" controls="" autoplay=""><track src="'.$pathurlencoded.'.vtt" kind="subtitles" default>Your browser doesn\'t support this format. Try Firefox.</video>[<a title="This stream has soft subtitles displayed in HTML5. Click to download the VTT file" href="'.$pathurlencoded.'.vtt">subs</a>]';
			else
				$mediacode .= "\t\t\t" . '<video id="media" src="' . $pathurlencoded  .'" controls="" autoplay="">Your browser doesn\'t support this format. Try Firefox.</video>';
			break;
		case 'opus':
			$mediacode .= "\t\t\t" . '<audio id="media" src="' . $pathurlencoded  .'" controls="" autoplay="">Your browser doesn\'t support this format. Try Firefox.</audio><span id="looplink"><a href="#" title="Loop play" onclick="looping()">∞</a></span>';
			break;
		case 'ogg':
			$mediacode .= "\t\t\t" . '<audio id="media" src="' . $pathurlencoded  .'" controls="" autoplay="">Your browser doesn\'t support this format. Try Firefox.</audio><span id="looplink"><a href="#" title="Loop play" onclick="looping()">∞</a></span>';
			break;
		case 'swf':
			$mediacode .= "\t\t\t" . '<embed src="' . $pathurlencoded  .'" quality="high" pluginspage="http://www.macromedia.com/go/getflashplayer" type="application/x-shockwave-flash">';
			break;
		case 'pdf':
			$mediacode .= "\t\t\t" . '<embed src="' . $pathurlencoded  .'#view=Fit">';
			break;
		case 'jpg':
			$mediacode .= "\t\t\t" . '<a title="' . $mediatitle . '" href="' . $pathurlencoded . '"><img id="media" alt="' . $mediatitle . '" src="' . $pathurlencoded . '"/></a>';
			break;
		case 'jpeg':
			$mediacode .= "\t\t\t" . '<a title="' . $mediatitle . '" href="' . $pathurlencoded . '"><img id="media" alt="' . $mediatitle . '" src="' . $pathurlencoded . '"/></a>';
			break;
		case 'png':
			$mediacode .= "\t\t\t" . '<a title="' . $mediatitle . '" href="' . $pathurlencoded . '"><img id="media" alt="' . $mediatitle . '" src="' . $pathurlencoded . '"/></a>';
			break;
		case 'webp':
			$mediacode .= "\t\t\t" . '<a title="' . $mediatitle . '" href="' . $pathurlencoded . '"><img id="media" alt="' . $mediatitle . '" src="' . $pathurlencoded . '"/></a>';
			break;
		case 'svg':
			$mediacode .= "\t\t\t" . '<a title="' . $mediatitle . '" href="' . $pathurlencoded . '"><img id="media" alt="' . $mediatitle . '" src="' . $pathurlencoded . '"/></a>';
			break;
		case 'gif':
			$mediacode .= "\t\t\t" . '<a title="' . $mediatitle . '" href="' . $pathurlencoded . '"><img id="media" alt="' . $mediatitle . '" src="' . $pathurlencoded . '"/></a>';
			break;
		default:
        		header("HTTP/1.1 404 File not found");
        		print "<h1>404 File not found</h1>";
        		exit(1);
	}
    } 
}

/* Below, the main HTML part. */
$mediacode = (isset($mediacode)) ? $mediacode : '';
$mediatitle = (isset($mediatitle)) ? $mediatitle : 'Home';
$stylefile = (file_exists('style.css')) ? 'style.css' : 'style.default.css';
print <<<EOT
<!DOCTYPE html>
<head>
    <meta charset="UTF-8">
    <title>{$conf['title']} – $mediatitle</title>
    <meta name="description" content="{$mediatitle} – {$conf['desc']}">
    <link rel='stylesheet' href='{$stylefile}' type='text/css' media='screen' />
<meta name="viewport" content="initial-scale=1.0, user-scalable=yes">
    <link rel="icon" type="image/png" href="favicon.png" />
    <link rel="alternate" type="application/atom+xml" title="ATOM last uploaded files" href="{$conf['uri']}/?feed" />
    <meta name="robots" content="index" />
</head>
<body>
<div id="header">
    {$conf['header']}
</div>
<div id="content">
    <div style="min-height:150px;">
        <h1><a href="./">{$conf['title']}</a></h1>
        <span style="font-size:small">{$conf['desc']}</span><br>
        <div id="mediacode">
            $mediacode
        </div>
    </div>

EOT;
        /* User can choose to sort files. */
        $options = '';
        if (isset($_GET['sort']) && $_GET['sort'] == 'asc') {
            $options .= '<option value="?sort=asc" selected="">Ascending</option>' ."\n";
        } else {
            $options .= '<option value="?sort=asc">Ascending</option>' ."\n";
        } if (isset($_GET['sort']) && $_GET['sort'] == 'mtime') {
            $options .= "\t\t\t" . '<option value="?sort=mtime" selected="">Last uploaded files</option>';
        } else {
            $options .= "\t\t\t" . '<option value="?sort=mtime">Last uploaded files</option>';
        }
        print <<<EOT

    <small>
        Sort by:
        <select onChange="if (this.value) window.location.href=this.value">
            $options
        </select><br />
        Download the <a href="?playlist">playlist</a> in XSPF format.<br />
        You can also download the <a href="?playlist&amp;sort=mtime">playlist</a> sorted by "last uploaded files".
    </small>
    <br />

EOT;
        /* Construct the "explorer". */
        print explorerHTML();
        print <<<EOT

</div>
    <div id="footer">
        {$conf['footer']}
    </div>
</body>
</html>
EOT;
